'Filename   ILI9341-Slave_100.bas
'Version    1.00
'Date       2019-03-25
'Purpose    Arduino UNO shield with ILI9341 320x240 Pixel, 8bit mode
'           No SD card, no touch screen
'           Landscape and Portrait
'           Demo landscape, text & graphic functions
'Author     Georg Latzel, DL6GL
'           ILI9341 driver from Hkipnik@aol.com, Date 01.05.2015
'Copyright  (c) Georg Latzel, free for private use under GNU license
'           ILI9341 driver: Hkipnik@aol.com
'           ILI9341 driver modifications:
'           -  ILI9341 declaration8.inc inserted into code here
'           -  ILI9341_functions8.inc modified
'              "Res_disp" code lines working on A4(PC4) disabled in Lcd_INIT
'              SD card functions removed
'              now "ILI9341_functions8_DL6GL.inc"
'              Only 2 small fonts are used to save memory:
'              (1)My6x8.font
'              (2)My12x16.font
'           -  Colors reduced
'Hardware   Arduino UNO, ATmega328p, Crystal 16 MHz
'           No name 2.8'' TFT UNO shield made in China
'           LCD D0   PB.0  LCD Data bit 0
'           LCD D1   PB.1  LCD Data bit 1
'           LCD D2   PD.2  LCD Data bit 2
'           LCD D3   PD.3  LCD Data bit 3
'           LCD D4   PD.4  LCD Data bit 4
'           LCD D5   PD.5  LCD Data bit 5
'           LCD D6   PD.6  LCD Data bit 6
'           LCD D7   PD.7  LCD Data bit 7
'           LCD RD   PC.0  LCD Read, set output high
'           LCD WR   PC.1  LCD Write
'           LCD RS   PC.2  LCD Register select
'           LCD CS   PC.3  LCD Chip select
'           LCD RST  PC.4  LCD Reset --> Removed for I2C SDA!!!!!
'                          TFT shield pin LCD-RST to Arduino Pin A4 removed.
'                          A4 = PC.4 is used for I2C-SDA.
'                          TFT shield pin LCD-RST connected to Arduino Reset pin
'                          via 10k, so LCD is resetted by Arduino Reset.
'                          Is on the same side of the shield, last connector
'                          next to 3.3V connector.
'                          Software LCD reset with Res_disp (Alias Portc.4)
'                          in ILI9341_functions8.inc is disabled.
'           SD SS    PB.2  SD card slave select
'                          Now used for sync signal to master
'           CD DI    PB.3  SD card data in      (not used, PB.3 is MOSI)
'           SD D0    PB.4  SD card data out     (not used, PB.4 is MISO)
'           SD SCK   PB.5  SD card serial clock (not used, PB.5 is SCK)
'           Power consumption Arduino UNO + TFT shield: 7V/~300mA
'
'           I2C communication with master controller
'           via ports C.4 (SDA) & C.5 (SCL).
'           I2C protocoll:
'           Byte 0: bytGD_Addr      Slave address
'           Byte 1: bytGD_Len       Number of data bytes from byte 1 to n
'           Byte 2: bytGD_Cmd       GLCD command
'           Byte 3: bytGD_Dat(1)    First data byte
'           ...
'           Byte n: bytGD_Dat(n-2)  Last data byte
'
'Compiler   BASCOM-AVR 2.0.7.6
'           For readable "camel case" variables mark "Don't change case"
'           in BASCOM Options-Programmer-Environment-Editor
'Edit history
'           2019-03-25, first release
'           Flash used: 15,290 bytes (46%)
'
'******************************************************************************
'FUSE-Bits (AVR Studio 4.0-Settings, [ ]=no check, [x]=check)
'In ATMEGA-Data sheets: Bit = 1 -> not set = no check
'                       Bit = 0 -> set     = check
'BODLEVEL   Brown-out detection 4.3V
'RSTDISBL   [ ]
'DWEN       [ ]
'SPIEN      [x] Do not change!
'WDTON      [ ]
'EESAVE     [ ]
'BOOTSZ     Boot Flash size=256 words (minimal boot size)
'BOOTRST    [x]
'CKDIV8     [ ]
'CKOUT      [ ]
'SUT_CKSEL  Ext. Crystal Osc. 8.0- MHz, Start-up time 16k + 14ms
'Extended:  0xFC
'High:      0xDE
'Low:       0xFF


$regfile = "m328pdef.dat"                         'ATMega328P
$crystal = 16000000                               '16 MHz xtal
$hwstack = 60
$swstack = 100
$framesize = 100

Const strVers = "V0.13"                           'Version number

'Configure ILI9341 data lines on PortB.0 to B.2 and D.2 to D.7 ----------------
'ILI9341 data lines on PortB.0 to B.5
DDRB == &B000111111                               'PortB.0...B.5 = output
Data_disp_b Alias Portb                           'PortB.0, B.1 = LCD D0, LCD D1
LCD_Busy Alias Data_disp_b.2                      'Port B.2 = LCD busy signal
Ddrd = &B11111100                                 'Port D.2...D.7=output
Data_disp_d Alias Portd                           'LCD D2...D7

'Configure ILI9341 pins on PortC.0 to C.3 -------------------------------------
'PortC.4, C.5 are used for I2C
Port_wr Alias Portc                               'Used in ILI9341_functions8
DDRC.0 = 1                                        'PortC.0 = LCD RD = output
PortC.0 = 1                                       'Set LCD RD output high
DDRC.1 = 1                                        'PortC.1 = LCD WR = output
Wr_disp Alias Portc.1                             'LCD WR
DDRC.2 = 1                                        'PortC.2 = LCD RS = output
Rs_disp Alias Portc.2                             'LCD RS
DDRC.3 = 1                                        'PortC.2 = LCD CS = output
Cs_disp Alias Portc.3                             'LCD CS
'Res_disp Alias Portc.4                            'LCD Reset -> Removed!!
'Config Res_disp = Output                          'Removed!!
Const Wrpin = 1                                   'WR Pin Nr
Wr_disp = 1
Cs_disp = 1

'Configure I2C on Port C ------------------------------------------------------
'Must be connected to Arduino UNO ports C.4 (SDA) & C.5 (SCL)
Config SDA = PortC.4                              'TWI SDA (hardware port)
Config SCL = PortC.5                              'TWI SCL (hardware port)

'Variables TWI-Communication with graphics controller -------------------------
Const bytTWIAdr = &H74                            'TWI Slave address
Dim bytGD_Len As Byte                             'Length of TWI message
Dim bytGD_Cmd As Byte                             'LCD command
Dim bytTWI_Buff(72) As Byte                       'TWI input buffer
Dim bytTWI_Count As Byte                          'TWI receive buffer byte counter
Dim bytTWI_control As Byte                        'TWI Control register data
Dim bytTWI_status As Byte                         'TWI status

'GLCD Variables ---------------------------------------------------------------
Dim bytFontNum As Byte
Dim bytBold As Byte
Dim bytFill As Byte
Dim bytWidth As Byte
Dim wrdBackColor As Word
Dim bytBackColor(2) As Byte At wrdBackColor Overlay
Dim wrdForeColor As Word
Dim bytForeColor(2) As Byte At wrdForeColor Overlay
Dim wrdXPos As Word
Dim bytXpos(2) As Byte At wrdXPos Overlay
Dim wrdYPos As Word
Dim bytYpos(2) As Byte At wrdYPos Overlay
Dim wrdXPos2 As Word
Dim bytXpos2(2) As Byte At wrdXPos2 Overlay
Dim wrdYPos2 As Word
Dim bytYpos2(2) As Byte At wrdYPos2 Overlay
Dim wrdXLen As Word
Dim bytXLen(2) As Byte At wrdXLen Overlay
Dim wrdYLen As Word
Dim bytYLen(2) As Byte At wrdYLen Overlay
Dim wrdRadius As Word
Dim bytRadius(2) As Byte At wrdRadius Overlay

Const TextLen = 50
Dim strText As String * TextLen
Dim bytText(TextLen) As Byte At strText Overlay
Dim Rgb16 As Word
Dim Rgb16_low As Byte At Rgb16 Overlay
Dim Rgb16_high As Byte At Rgb16 + 1 Overlay
Dim Hoehe As Word
Dim Breit As Word
Dim Zeichenarray(TextLen) As Byte
Dim strChar1 As String * 1

'Common variables, multiply used ----------------------------------------------
Dim bytTmp0 As Byte
Dim bytTmp1 As Byte

'ILI9341 Display mode, 1=Portrait 2=Portrait 180 3=landscape 4=landscape 180-
Const Lcd_mode = 3                                'Is Landscape
#if Lcd_mode = 1
  Const X_size = 239
  Const Y_size = 319
#elseif Lcd_mode = 2
  Const X_size = 239
  Const Y_size = 319
#elseif Lcd_mode = 3
  Const X_size = 319
  Const Y_size = 239
#elseif Lcd_mode = 4
  Const X_size = 319
  Const Y_size = 239
#endif

'Colors -----------------------------------------------------------------------
Const Transparent = &H1000
'Const Aliceblue = &HF7DF
'Const Antiquewhite = &HFF5A
'Const Aqua = &H07FF
'Const Aquamarine = &H7FFA
'Const Azure = &HF7FF
Const Beige = &HF7BB
'Const Bisque = &HFF38
Const Black = &H0000
'Const Blanchedalmond = &HFF59
Const Blue = &H001F
'Const Blueviolet = &H895C
Const Brown = &HA145
'Const Burlywood = &HDDD0
'Const Cadetblue = &H5CF4
'Const Chartreuse = &H7FE0
'Const Chocolate = &HD343
'Const Coral = &HFBEA
'Const Cornflowerblue = &H64BD
'Const Cornsilk = &HFFDB
'Const Crimson = &HD8A7
Const Cyan = &H07FF
Const Darkblue = &H0011
Const Darkcyan = &H0451
'Const Darkgoldenrod = &HBC21
Const Darkgray = &HAD55
Const Darkgreen = &H0320
'Const Darkkhaki = &HBDAD
'Const Darkmagenta = &H8811
'Const Darkolivegreen = &H5345
'Const Darkorange = &HFC60
'Const Darkorchid = &H9999
Const Darkred = &H8800
'Const Darksalmon = &HECAF
'Const Darkseagreen = &H8DF1
'Const Darkslateblue = &H49F1
'Const Darkslategray = &H2A69
'Const Darkturquoise = &H067A
'Const Darkviolet = &H901A
'Const Deeppink = &HF8B2
'Const Deepskyblue = &H05FF
Const Dimgray = &H6B4D
'Const Dodgerblue = &H1C9F
'Const Firebrick = &HB104
'Const Floralwhite = &HFFDE
'Const Forestgreen = &H2444
'Const Fuchsia = &HF81F
'Const Gainsboro = &HDEFB
'Const Ghostwhite = &HFFDF
Const Gold = &HFEA0
'Const Goldenrod = &HDD24
Const Gray = &H8410
Const Green = &H07E0
'Const Green1 = &H0700
Const Greenyellow = &HAFE5
'Const Honeydew = &HF7FE
'Const Hotpink = &HFB56
'Const Indianred = &HCAEB
Const Indigo = &H4810
'Const Ivory = &HFFFE
'Const Khaki = &HF731
'Const Lavender = &HE73F
'Const Lavenderblush = &HFF9E
'Const Lawngreen = &H7FE0
'Const Lemonchiffon = &HFFD9
Const Lightblue = &HAEDC
'Const Lightcoral = &HF410
'Const Lightcyan = &HE7FF
'Const Lightgold = &HFFDA
'Const Lightgreen = &H9772
Const Lightgrey = &HD69A
'Const Lightpink = &HFDB8
'Const Lightsalmon = &HFD0F
'Const Lightseagreen = &H2595
'Const Lightskyblue = &H867F
'Const Lightslategray = &H7453
'Const Lightsteelblue = &HB63B
Const Lightyellow = &HFFFC
'Const Lime = &H07E0
'Const Limegreen = &H3666
'Const Linen = &HFF9C
Const Magenta = &HF81F
'Const Maroon = &H8000
'Const Mediumaquamarine = &H6675
'Const Mediumblue = &H0019
'Const Mediumorchid = &HBABA
'Const Mediumpurple = &H939B
'Const Mediumseagreen = &H3D8E
'Const Mediumslateblue = &H7B5D
'Const Mediumspringgreen = &H07D3
'Const Mediumturquoise = &H4E99
'Const Mediumvioletred = &HC0B0
'Const Midnightblue = &H18CE
'Const Mintcream = &HF7FF
'Const Mistyrose = &HFF3C
'Const Moccasin = &HFF36
'Const Navajowhite = &HFEF5
'Const Navy = &H0010
'Const Oldlace = &HFFBC
'Const Olive = &H8400
'Const Olivedrab = &H6C64
Const Orange = &HFD20
Const Orangered = &HFA20
'Const Orchid = &HDB9A
'Const Palegoldenrod = &HEF55
'Const Palegreen = &H9FD3
'Const Paleturquoise = &HAF7D
'Const Palevioletred = &HDB92
'Const Papayawhip = &HFF7A
'Const Peachpuff = &HFED7
'Const Peru = &HCC27
'Const Plum = &HDD1B
'Const Powderblue = &HB71C
'Const Purple = &H8010
Const Red = &HF800
'Const Rosybrown = &HBC71
'Const Royalblue = &H435C
'Const Saddlebrown = &H8A22
'Const Salmon = &HFC0E
'Const Sandybrown = &HF52C
'Const Seagreen = &H2C4A
'Const Seashell = &HFFBD
'Const Sienna = &HA285
Const Silver = &HC618
'Const Skyblue = &H867D
'Const Slateblue = &H6AD9
'Const Slategray = &H7412
'Const Snow = &HFFDF
'Const Springgreen = &H07EF
'Const Steelblue = &H4416
'Const Teal = &H0410
'Const Thistle = &HDDFB
'Const Tomato = &HFB08
'Const Turquoise = &H471A
Const Violet = &HEC1D
'Const Wheat = &HF6F6
Const White = &HFFFF
'Const Whitesmoke = &HF7BE
Const Yellow = &HFFE0
Const Yellowgreen = &H9E66
Const Gray1 = &HC618
Const Gray2 = &HA514
Const Gray3 = &H630C
Const Gray4 = &H4208
Const Gray5 = &H2104
Const Gray6 = &H3186

'GLCD Subs --------------------------------------------------------------------
Declare Sub Lcd_init()
Declare Sub Lcd_write_data(byval Daten As Word)
Declare Sub Lcd_select_reg(byval Index As Byte )
Declare Sub Lcd_write_byte(byval Daten As Byte)
Declare Sub Lcd_set_window(byval Xpos As Word , Byval Ypos As Word , Byval Xsize As Word , Byval Ysize As Word)
Declare Sub Lcd_set_pixel(byval X1 As Word , Byval Y1 As Word , Byval Color As Word)
Declare Sub Lcd_line(byval X1 As Word , Byval Y1 As Word , Byval X2 As Word , Byval Y2 As Word , Byval Pen_width As Byte , Byval Color As Word)
Declare Sub Lcd_box(byval Xstart As Word , Byval Ystart As Word , Byval X_length As Word , Byval Y_height As Word , Byval Fill As Byte , Byval Color As Word , Byval Bordercolor As Word)
Declare Sub Lcd_fill_circle(byval X As Word , Byval Y As Word , Byval Radius As Word , Byval Color1 As Word)
Declare Sub Lcd_circle(byval X As Word , Byval Y As Word , Byval Radius As Word , Byval Color As Word)
Declare Sub Lcd_clear(byval Color As Word)
Declare Sub Lcd_text(byval Text As String , Byval Lcd_posx As Word , Byval Lcd_posy As Word , Byval Fontset As Byte , Byval Forecolor As Word , Byval Backcolor As Word , Byval Bold As Byte)

'I2C Subs ---------------------------------------------------------------------
Declare Sub TWISlaveInit
Declare Sub TWI_GetData

'Enable TWI interrupt service routine -----------------------------------------
On TWI TWI_isr                                    'Go to TWI_isr if character is received
Call TWISlaveInit                                 'Initialize TWI slave
Enable TWI                                        'Enable TWI receive interrupt
SREG.7 = 1                                        'Enable all interrupts

Call Lcd_init                                     'Init display
LCD_Busy = 0                                      'LCD is not busy

Do

   Call TWI_GetData                               'Watch TWI & read Master data

Loop

'==============================================================================
Sub TWI_GetData

    If bytTWI_Count > 0 And bytTWI_Count = bytGD_Len Then       'Last byte: Finished
      bytGD_Cmd = bytTWI_Buff(2)
      Select Case bytGD_Cmd
         Case 1
            'Call Lcd_init                         'Init display

         Case 2                                   'GLCD_Clear
            bytBackColor(1) = bytTWI_Buff(3)      'Color lo byte
            bytBackColor(2) = bytTWI_Buff(4)      'Color Hi byte
            LCD_Busy = 1                          'LCD is busy
            Call Lcd_clear(wrdBackColor)

         Case 10                                  'GLCD_Text
            bytXPos(1) = bytTWI_Buff(3)           'X pos lo byte
            bytXPos(2) = bytTWI_Buff(4)           'X pos hi byte
            bytYPos(1) = bytTWI_Buff(5)           'Y pos lo byte
            bytYPos(2) = bytTWI_Buff(6)           'Y pos hi byte
            bytFontNum = bytTWI_Buff(7)           'Font number
            bytForeColor(1) = bytTWI_Buff(8)      'Color lo byte
            bytForeColor(2) = bytTWI_Buff(9)      'Color hi byte
            bytBackColor(1) = bytTWI_Buff(10)     'Color lo byte
            bytBackColor(2) = bytTWI_Buff(11)     'Color hi byte
            bytBold = bytTWI_Buff(12)             'Bold index
            strText = ""
            For bytTmp0 = 13 To bytGD_Len         'Create string
               bytTmp1 = bytTWI_Buff(bytTmp0)     'ASCII code
               strChar1 = Chr(bytTmp1)            'converted to character
               strText = strText + strChar1       'make string
            Next bytTmp0
            LCD_Busy = 1                          'LCD is busy
            Call Lcd_text(strText , wrdXPos , wrdYpos , bytFontNum , wrdForecolor , wrdBackColor , bytBold)

         Case 20                                  'GLCD_Box
            bytXPos(1) = bytTWI_Buff(3)           'X pos lo byte
            bytXPos(2) = bytTWI_Buff(4)           'X pos hi byte
            bytYPos(1) = bytTWI_Buff(5)           'Y pos lo byte
            bytYPos(2) = bytTWI_Buff(6)           'Y pos hi byte
            bytXLen(1) = bytTWI_Buff(7)           'X len lo byte
            bytXLen(2) = bytTWI_Buff(8)           'X len hi byte
            bytYLen(1) = bytTWI_Buff(9)           'Y len lo byte
            bytYLen(2) = bytTWI_Buff(10)          'Y len hi byte
            bytFill = bytTWI_Buff(11)             'bytFill
            bytBackColor(1) = bytTWI_Buff(12)     'Color lo byte
            bytBackColor(2) = bytTWI_Buff(13)     'Color hi byte
            bytForeColor(1) = bytTWI_Buff(14)     'Color lo byte
            bytForeColor(2) = bytTWI_Buff(15)     'Color hi byte
            LCD_Busy = 1                          'LCD is busy
            Call Lcd_Box(wrdXPos , wrdYPos , wrdXLen , wrdYLen , bytFill , wrdBackColor , wrdForeColor)

         Case 30                                  'GLCD circle
            bytXPos(1) = bytTWI_Buff(3)           'X pos lo byte
            bytXPos(2) = bytTWI_Buff(4)           'X pos hi byte
            bytYPos(1) = bytTWI_Buff(5)           'Y pos lo byte
            bytYPos(2) = bytTWI_Buff(6)           'Y pos hi byte
            bytRadius(1) = bytTWI_Buff(7)         'Radius lo byte
            bytRadius(2) = bytTWI_Buff(8)         'Radius hi byte
            LCD_Busy = 1                          'LCD is busy
            Call Lcd_circle(wrdXPos , wrdYPos , wrdRadius , wrdForecolor)

         Case 32                                  'GLCD circle
            bytXPos(1) = bytTWI_Buff(3)           'X pos lo byte
            bytXPos(2) = bytTWI_Buff(4)           'X pos hi byte
            bytYPos(1) = bytTWI_Buff(5)           'Y pos lo byte
            bytYPos(2) = bytTWI_Buff(6)           'Y pos hi byte
            bytRadius(1) = bytTWI_Buff(7)         'Radius lo byte
            bytRadius(2) = bytTWI_Buff(8)         'Radius hi byte
            bytBackColor(1) = bytTWI_Buff(9)      'Back color lo byte
            bytBackColor(2) = bytTWI_Buff(10)     'Back color hi byte
            LCD_Busy = 1                          'LCD is busy
            Call Lcd_fill_circle(wrdXPos , wrdYPos , wrdRadius , wrdBackcolor)

         Case 40                                  'GLCD Line
            bytXPos(1) = bytTWI_Buff(3)           'X1 pos lo byte
            bytXPos(2) = bytTWI_Buff(4)           'X1 pos hi byte
            bytYPos(1) = bytTWI_Buff(5)           'Y1 pos lo byte
            bytYPos(2) = bytTWI_Buff(6)           'Y1 pos hi byte
            bytXPos2(1) = bytTWI_Buff(7)          'X2 pos lo byte
            bytXPos2(2) = bytTWI_Buff(8)          'X2 pos hi byte
            bytYPos2(1) = bytTWI_Buff(9)          'Y2 pos lo byte
            bytYPos2(2) = bytTWI_Buff(10)         'Y2 pos hi byte
            bytWidth = bytTWI_Buff(11)            'Line width
            bytForeColor(1) = bytTWI_Buff(12)     'Line color lo byte
            bytForeColor(2) = bytTWI_Buff(13)     'Line color hi byte
            LCD_Busy = 1                          'LCD is busy
            Call Lcd_line(wrdXPos , wrdYpos , wrdXPos2 , wrdYPos2 , bytwidth , wrdForeColor)

      End Select
      bytTWI_Count = 0
      LCD_Busy = 0                                'LCD is idle
    End If

End Sub

'==============================================================================
Sub TWISlaveInit
'Initialize TWI slave function
'Input:
'bytTWIAdr  TWI slave address

   TWSR = 0                                       'Clear TWI status register
   TWBR = 0                                       'Clear baud rate register
   TWDR = &B11111111                              'TWI data register initial value
   TWAR = bytTWIAdr                               'TWI slave address
   TWCR = &B01000100                              'TWI control register
   '                                              'Bit 2: TWEN, enable TWI
   '                                              'Bit 6: TWEA, enable Ack
End Sub

'==============================================================================
TWI_isr:
'TWI receive interrupt service routine
'Success is checked with TWI status register TWSR
'Status code is stored in the 5 upper bits, masked out with "And &B11111000"
'For status codes see AVR data sheet
'Output: bytTWI_Buff(bytTWI_Count)  Received data bytes
'        bytTWI_Count               Byte counter
'        bytGD_Len                  = bytTWI_Buff(1): number of received bytes

      bytTWI_Status = TWSR And &B11111000         'Status register, mask TWI status=bits 7...3
      If bytTWI_Status = &HA0 Then                'Stop or repeated start condition
         bytTWI_Count = 0                         'Initialize byte counter
      ElseIf bytTWI_Status = &H60 Or bytTWI_Status = &H68 Then       'Control data byte received
         bytTWI_Count = 0                         'Initialize byte counter
      Elseif bytTWI_Status = &H80 Or bytTWI_Status = &H88 Then       'Data received
         Incr bytTWI_Count                        'Increment byte counter
         bytTWI_Buff(bytTWI_Count) = TWDR         'Received byte into input buffer
         If bytTWI_Count = 1 Then                 'Is data length
            bytGD_Len = bytTWI_Buff(1)            'Total transferred data length
         End If
      End If
      TWCR = &B11000101                           'Set TWI Control register
                                                  'Bit 0: TWIE , enable TWI interrupt
                                                  'Bit 2: TTWEN, TWI enable
                                                  'Bit 6: TWAE , TWI enable acknowledge bit
                                                  'Bit 7: TWINT, TWI interrupt flag
Return

'==============================================================================

'ILI9341 driver functions -----------------------------------------------------
$include "ILI9341_functions8_DL6GL.inc"

End

'Include fonts, must be after End statement -----------------------------------
$include "Font\My6x8.font"                        'Font #1
$include "Font\My12x16.font"                      'Font #2